﻿using SFML.Graphics;
using SFML.System;
using System;

//====================================================
// Описание работы классов и методов исходника на:
// www.interestprograms.ru
// Исходные коды программ и игр
// Автор исходных кодов Клочков Павел
//====================================================

namespace sfml4
{
    class Asteroid
    {
        // Скорость разлетания детонации
        private readonly float velocityDetonation = 20.0f;

        // Скорость разлетания от взрыва после детонации
        private readonly float velocityExplosion = 0.5f;

        // Скорость вращения
        private readonly float velocityRotation = 4;

        // Флаг показа детонации
        private bool showDetonation = false;

        // Флаг показа последствия взрыва
        private bool showExplosion = false;

        // Ротация частей астероида после взрыва
        private readonly float[] rotationExplosion = new float[5];

        // Вид астероида
        private readonly Texture[] texture =
        {
            new SFML.Graphics.Texture(Resource.asteroid_p1),
            new SFML.Graphics.Texture(Resource.asteroid_p2),
            new SFML.Graphics.Texture(Resource.asteroid_p3),
            new SFML.Graphics.Texture(Resource.asteroid_p4),
            new SFML.Graphics.Texture(Resource.asteroid_p5)
        };

        // Функциональность астероида
        private readonly Sprite[] sprite = new Sprite[5];

        // Случайные числа для естественности взрыва
        private readonly Random Random = new Random();

        public Asteroid()
        {
            for (int i = 0; i < texture.Length; i++)
            {
                sprite[i] = new Sprite(texture[i])
                {
                    Origin = new Vector2f(texture[i].Size.X / 2, texture[i].Size.Y / 2),
                };
            }

            // Первоначальная позиция
            Position(new Vector2f(-300, 0));
        }




        public void Draw(RenderWindow window)
        {
            // Детонация
            if (showDetonation == true)
            {
                showDetonation = false;

                ShowExplosion(velocityDetonation);
            }

            // Последствия взрыва
            if (showExplosion == true)
            {
                ShowExplosion(velocityExplosion);
            }


            // Астероид стандартный
            for (int i = 0; i < texture.Length; i++)
            {
                window.Draw(sprite[i]);
            }
        }

        public void Position(Vector2f v)
        {
            showDetonation = false;
            showExplosion = false;

            for (int i = 0; i < texture.Length; i++)
            {
                sprite[i].Position = v;

                sprite[i].Rotation = 0;
            }
        }

        // 
        public void StartExplosion()
        {
            showDetonation = true;
            showExplosion = true;

            for (int i = 0; i < rotationExplosion.Length; i++)
            {
                rotationExplosion[i] = Random.Next(-30, 30) / 100.0f * velocityRotation;
            }
        }


        // Астероид. Симуляция взрыва. Разлетаются осколки
        // Осколки разлетаются, вращаясь, в разные стороны
        private void ShowExplosion(float velocity)
        {
            // Векторы направления

            // 1 летит влево-вверх
            int x = Random.Next(-250, 20);
            int y = Random.Next(-150, -10);

            var direction = System.Numerics.Vector2.Normalize(new System.Numerics.Vector2(x, y));
            var temp = new Vector2f(direction.X * velocity, direction.Y * velocity);
            sprite[0].Position += temp;


            // 2 летит влево-вниз
            x = Random.Next(-250, 20);
            y = Random.Next(10, 150);


            direction = System.Numerics.Vector2.Normalize(new System.Numerics.Vector2(x, y));
            temp = new Vector2f(direction.X * velocity, direction.Y * velocity);
            sprite[1].Position += temp;


            // 3 летит вниз
            x = Random.Next(-50, 50);
            y = Random.Next(10, 250);


            direction = System.Numerics.Vector2.Normalize(new System.Numerics.Vector2(x, y));
            temp = new Vector2f(direction.X * velocity, direction.Y * velocity);
            sprite[2].Position += temp;


            // 4 летит вправо-вниз
            x = Random.Next(20, 250);
            y = Random.Next(10, 250);


            direction = System.Numerics.Vector2.Normalize(new System.Numerics.Vector2(x, y));
            temp = new Vector2f(direction.X * velocity, direction.Y * velocity);
            sprite[3].Position += temp;


            // 5 летит вправо-вверх
            x = Random.Next(-20, 250);
            y = Random.Next(-250, -50);

            direction = System.Numerics.Vector2.Normalize(new System.Numerics.Vector2(x, y));
            temp = new Vector2f(direction.X * velocity, direction.Y * velocity);
            sprite[4].Position += temp;

            for (int i = 0; i < sprite.Length; i++)
            {
                sprite[i].Rotation += rotationExplosion[i];
            }

        }

    }
}
